package javax.swing.text.html.parser;

import java.io.DataInputStream;
import sun.awt.AppContext;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.BitSet;
import java.util.Hashtable;
import java.util.Vector;

public class DTD implements DTDConstants
{
    public String name;
    public Vector<Element> elements;
    public Hashtable<String, Element> elementHash;
    public Hashtable<Object, Entity> entityHash;
    public final Element pcdata;
    public final Element html;
    public final Element meta;
    public final Element base;
    public final Element isindex;
    public final Element head;
    public final Element body;
    public final Element applet;
    public final Element param;
    public final Element p;
    public final Element title;
    final Element style;
    final Element link;
    final Element script;
    public static final int FILE_VERSION = 1;
    private static final Object DTD_HASH_KEY;
    
    protected DTD(String name) {
        this.elements = new Vector();
        this.elementHash = new Hashtable();
        this.entityHash = new Hashtable();
        this.pcdata = this.getElement("#pcdata");
        this.html = this.getElement("html");
        this.meta = this.getElement("meta");
        this.base = this.getElement("base");
        this.isindex = this.getElement("isindex");
        this.head = this.getElement("head");
        this.body = this.getElement("body");
        this.applet = this.getElement("applet");
        this.param = this.getElement("param");
        this.p = this.getElement("p");
        this.title = this.getElement("title");
        this.style = this.getElement("style");
        this.link = this.getElement("link");
        this.script = this.getElement("script");
        this.name = name;
        this.defEntity("#RE", 65536, 13);
        this.defEntity("#RS", 65536, 10);
        this.defEntity("#SPACE", 65536, 32);
        this.defineElement("unknown", 17, false, true, null, null, null, null);
    }
    
    public String getName() {
        return this.name;
    }
    
    public Entity getEntity(String key) {
        return (Entity)this.entityHash.get(key);
    }
    
    public Entity getEntity(int i) {
        return (Entity)this.entityHash.get(Integer.valueOf(i));
    }
    
    boolean elementExists(String s) {
        return !"unknown".equals(s) && this.elementHash.get(s) != null;
    }
    
    public Element getElement(String s) {
        Element element = (Element)this.elementHash.get(s);
        if (element == null) {
            element = new Element(s, this.elements.size());
            this.elements.addElement(element);
            this.elementHash.put(s, element);
            return element;
        }
        return element;
    }
    
    public Element getElement(int index) {
        return (Element)this.elements.elementAt(index);
    }
    
    public Entity defineEntity(String s, int n, char[] array) {
        Entity entity = (Entity)this.entityHash.get(s);
        if (entity == null) {
            entity = new Entity(s, n, array);
            this.entityHash.put(s, entity);
            if ((n & 0x10000) != 0x0 && array.length == 1) {
                switch (n & 0xFFFEFFFF) {
                    case 1:
                    case 11: {
                        this.entityHash.put(Integer.valueOf((int)array[0]), entity);
                        break;
                    }
                }
            }
        }
        return entity;
    }
    
    public Element defineElement(String s, int type, boolean oStart, boolean oEnd, ContentModel content, BitSet exclusions, BitSet inclusions, AttributeList atts) {
        Element element = this.getElement(s);
        element.type = type;
        element.oStart = oStart;
        element.oEnd = oEnd;
        element.content = content;
        element.exclusions = exclusions;
        element.inclusions = inclusions;
        element.atts = atts;
        return element;
    }
    
    public void defineAttributes(String s, AttributeList atts) {
        this.getElement(s).atts = atts;
    }
    
    public Entity defEntity(String s, int n, int n2) {
        return this.defineEntity(s, n, new char[] { (char)n2 });
    }
    
    protected Entity defEntity(String s, int n, String s2) {
        int length = s2.length();
        char[] dst = new char[length];
        s2.getChars(0, length, dst, 0);
        return this.defineEntity(s, n, dst);
    }
    
    protected Element defElement(String s, int n, boolean b, boolean b2, ContentModel contentModel, String[] array, String[] array2, AttributeList list) {
        BitSet set = null;
        if (array != null && array.length > 0) {
            set = new BitSet();
            for (String s2 : array) {
                if (s2.length() > 0) {
                    set.set(this.getElement(s2).getIndex());
                }
            }
        }
        Object o = null;
        if (array2 != null && array2.length > 0) {
            o = new BitSet();
            for (String s3 : array2) {
                if (s3.length() > 0) {
                    o.set(this.getElement(s3).getIndex());
                }
            }
        }
        return this.defineElement(s, n, b, b2, contentModel, set, o, list);
    }
    
    protected AttributeList defAttributeList(String s, int n, int n2, String s2, String str, AttributeList list) {
        Vector<String> vector = null;
        if (str != null) {
            vector = new Vector();
            StringTokenizer stringTokenizer = new StringTokenizer(str, "|");
            while (stringTokenizer.hasMoreTokens()) {
                String nextToken = stringTokenizer.nextToken();
                if (nextToken.length() > 0) {
                    vector.addElement(nextToken);
                }
                else {
                    continue;
                }
            }
        }
        return new AttributeList(s, n, n2, s2, vector, list);
    }
    
    protected ContentModel defContentModel(int n, Object o, ContentModel contentModel) {
        return new ContentModel(n, o, contentModel);
    }
    
    @Override
    public String toString() {
        return this.name;
    }
    
    public static void putDTDHash(String key, DTD value) {
        getDtdHash().put(key, value);
    }
    
    public static DTD getDTD(String lowerCase) throws IOException {
        lowerCase = lowerCase.toLowerCase();
        DTD dtd = (DTD)getDtdHash().get(lowerCase);
        if (dtd == null) {
            dtd = new DTD(lowerCase);
            return dtd;
        }
        return dtd;
    }
    
    private static Hashtable<String, DTD> getDtdHash() {
        AppContext appContext = AppContext.getAppContext();
        Hashtable<Object, Object> value = (Hashtable)appContext.get(DTD.DTD_HASH_KEY);
        if (value == null) {
            value = new Hashtable();
            appContext.put(DTD.DTD_HASH_KEY, value);
            return value;
        }
        return value;
    }
    
    public void read(DataInputStream dataInputStream) throws IOException {
        dataInputStream.readInt();
        String[] array = new String[dataInputStream.readShort()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = dataInputStream.readUTF();
        }
        for (int short1 = dataInputStream.readShort(), j = 0; j < short1; ++j) {
            this.defEntity(array[dataInputStream.readShort()], dataInputStream.readByte() | 0x10000, dataInputStream.readUTF());
        }
        for (int short2 = dataInputStream.readShort(), k = 0; k < short2; ++k) {
            int short3 = dataInputStream.readShort();
            int byte1 = dataInputStream.readByte();
            int byte2 = dataInputStream.readByte();
            this.defElement(array[short3], byte1, (byte2 & 0x1) != 0x0, (byte2 & 0x2) != 0x0, this.readContentModel(dataInputStream, array), this.readNameArray(dataInputStream, array), this.readNameArray(dataInputStream, array), this.readAttributeList(dataInputStream, array));
        }
    }
    
    private ContentModel readContentModel(DataInputStream dataInputStream, String[] array) throws IOException {
        switch (dataInputStream.readByte()) {
            case 0: {
                return null;
            }
            case 1: {
                return this.defContentModel((int)dataInputStream.readByte(), this.readContentModel(dataInputStream, array), this.readContentModel(dataInputStream, array));
            }
            case 2: {
                return this.defContentModel((int)dataInputStream.readByte(), this.getElement(array[dataInputStream.readShort()]), this.readContentModel(dataInputStream, array));
            }
            default: {
                throw new IOException("bad bdtd");
            }
        }
    }
    
    private String[] readNameArray(DataInputStream dataInputStream, String[] array) throws IOException {
        int short1 = dataInputStream.readShort();
        if (short1 == 0) {
            return null;
        }
        String[] array2 = new String[short1];
        for (int i = 0; i < short1; ++i) {
            array2[i] = array[dataInputStream.readShort()];
        }
        return array2;
    }
    
    private AttributeList readAttributeList(DataInputStream dataInputStream, String[] array) throws IOException {
        Object o = null;
        int short1;
        int byte1;
        int byte2;
        String s;
        Object o2;
        for (int i = dataInputStream.readByte(); i > 0; o = new AttributeList(array[short1], byte1, byte2, s, o2, o), --i) {
            short1 = dataInputStream.readShort();
            byte1 = dataInputStream.readByte();
            byte2 = dataInputStream.readByte();
            int short2 = dataInputStream.readShort();
            s = ((short2 == -1) ? null : array[short2]);
            o2 = null;
            int short3 = dataInputStream.readShort();
            if (short3 > 0) {
                o2 = new Vector(short3);
                for (int j = 0; j < short3; ++j) {
                    o2.addElement(array[dataInputStream.readShort()]);
                }
            }
        }
        return o;
    }
    
    static {
        DTD_HASH_KEY = new Object();
    }
}
